//=========================================================
// IAVRA_LocalizationCore.js
//=========================================================

/*:
@plugindesc The core plugin for localization, which contains all logic to load text and convert escape codes.
<Iavra Localization Core>
@author Iavra

@param Escape Code
@type text
@desc Code, that will be used to retrieve localized strings. "{key}" serves as placeholder for the text key. It's better leave it alone.
@default #{{key}}

@param Languages
@type text
@desc Comma-separated list of languages, that should be supported. The first entry will be used as the default.
@default en, ptbr, esp

@param File Path
@type text
@desc Path to the language files to load. The sequence "{lang}" will be replaced with the languages entered above. It's better leave it alone.
@default {lang}.json

@param Option Label
@type text
@desc Label to be used for the option in the options menu.
@default Language

@param Language Labels
@type text
@desc Comma-separated list of "language:label" pairs, to be used for the language selection.
@default en:English, ptbr:Portuguese

@help
============================================================================
Introduction
============================================================================

This plugin is a fusion of iavra_localization_core and iavra_localization
_menu into one. Also I did not change anything in the codes. I just changed 
the way it was done, but it's still the same thing.
In addition to the merging of plugins, the news was two bugs fixed by me and 
another already fixed by the author himself through a snippet posted by him 
on his thread. I also made some changes in the helpfile.

============================================================================
Features
============================================================================

• Add as many languages ​​as you want.
• Do your translations into an external .json file and import it into your 
project.
• You can also translate the database (Skills, actors name, equipment etc.)
• You can also translate texts inserted into parameters of other plugins.
• Change the language through the options menu or by script calls.
• Works on desktop and android versions.
 
============================================================================
How to use
============================================================================

First put the languages you want in the "Languages" parameters. You can put 
as many as you want.

To setup this plugin, register all supported languages inside the "Languages" parameter, separated by commas. You
also need to place the corresponding files inside your project folder. So, for example, if you want to support the
languages "en" (english) and "de" (german) and the parameter "File Path" is set to its default value, you'll need
to add the two files "de.json" and "en.json" to the root folder of your project.

The first language will automatically used as the default langage.

During runtime, every instance of "#{...}" (can be changed via plugin parameter "Escape Code") will get replaced
with a localized string, where "..." stands for a text key. So, if your language file looks like this:

{
"text.test": "This is a test text"
}

Every instance of "#{text.test}" will be replaced with "This is a test text". For better maintainability, it's also
possible to split keys at dots:

{
"text": {
"test": "This is a test text"
}
}

The above example will result in the same key as the first one, but makes it easier to construct nested keys, for
example for the names of all actors. Instead of objects, you can also use arrays, like this:

{
"text": ["Text 1", "Text 2", "Text 3"]
}

This will create the keys "text.0", "text.1" and "text.2", each pointing to the text listed at that array index. If
needed, you are free to combine the object and array notations.

Keys can also contain keys themselves, which will be replaced recursively. This allows you, to define important
strings, like city names, at a single position and reference it everywhere else:

{
"text": "Hi, my name is #{actor}",
"actor": "Harold"
}

You can use all escape characters, like "\V[...]" inside the files, but will need to double the backslashes. Line
breaks can be entered by using "\n", since JSON doesn't support real linebreaks inside strings.

Simply place this plugin in your plugin list after "Iavra Localization - Core". The parameters "Option Label" and
"Language Labels" are used to define the text to be shown. You are free to use escape codes inside those parameters,
as well, though i recommend against localizing the language labels.

The plugin offers the following script calls:

IAVRA.I18N.language;         Returns the current language.
IAVRA.I18N.language = lang;  Sets the current language to lang.
IAVRA.I18N.languages();      Returns a list of all available languages.
IAVRA.I18N.localize(text);   Localizes the given text.

============================================================================
Terms of Use
============================================================================
 
============================================================================
Special thanks and considerations
============================================================================
 
============================================================================
Contact
============================================================================
 
============================================================================
Updatelog
============================================================================

*/

"use strict";

var Imported = Imported || {};
Imported.IAVRA_LocalizationCore = true;

var IAVRA = IAVRA || {};
IAVRA.LocalizationCore = IAVRA.LocalizationCore || {};


IAVRA.Parameters = PluginManager.parameters('IAVRA_LocalizationCore');
IAVRA.Param = IAVRA.Param || {};

IAVRA.Param.LocalizationCore = {
EscapeCode: String(IAVRA.Parameters['Escape Code']),
IvCoreLanguages: String(IAVRA.Parameters['Languages']).split(/\s*,\s*/).filter(function(lang) { return !!lang; }),
IvCoreFilePath: String(IAVRA.Parameters['File Path']),
IvCoreLabelOption: String(IAVRA.Parameters['Option Label']),
IvCoreLabelLanguages: String(IAVRA.Parameters['Language Labels']).split(/\s*,\s*/).reduce(function(map, entry) {
var split = entry.split(':'); 
if(split.length === 2) { 
map[split[0]] = split[1]; 
} 
return map;
}, {})
};

/**
* Regex used to replace escape codes with localized data.
*/
var IvCoreRegex = new RegExp(IAVRA.Param.LocalizationCore.EscapeCode.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&").replace('\\{key\\}', '([\\w\\.]+?)'), 'g');

/**
* The current language.
*/
var IvCoreLang = IAVRA.Param.LocalizationCore.IvCoreLanguages[0];

/**
* Contains all localized data read from the data files defined "File Path". We intentionally don't use the object
* literal {}, but create a new Object with a null prototype, so we don't inherit any properties.
*/
var IvCoreData = IAVRA.Param.LocalizationCore.IvCoreLanguages.reduce(function(map, lang) { map[lang] = null; return map; }, Object.create(null));

/**
* Indicated, whether we are currently inside of "drawTextEx". Not thread-/async-safe!
*/
var IvCoreinDrawTextEx = false;

/**
* Initializes the language data by reading all files and storing their content inside the _data object. The
* undefined data serves as fallback, if no language is given in the "Languages" plugin parameter.
*/
function IvCoreInitialize() {
IAVRA.Param.LocalizationCore.IvCoreLanguages.forEach(function(lang) {
IvCoreLoadFile(IAVRA.Param.LocalizationCore.IvCoreFilePath.replace('{lang}', lang), function(data) { 
IvCoreData[lang] = IvCoreFlatten(data); });
});
IvCoreData[undefined] = {};
};


/**
* Loads a JSON file and executes the given callback with the parsed file contents as a parameter.
*/
function IvCoreLoadFile(url, callback) {
var request = new XMLHttpRequest();
request.open('GET', url);
request.overrideMimeType('application/json');
request.onload = function() { 
callback(JSON.parse(request.responseText));
};
request.onerror = function() { 
throw new Error("There was an error loading the file '" + url + "'."); 
};
request.send();
};

/**
* Flattens the given data object by joining nested object keys and array indizes with "." and returns a single-
* level object, whose keys can be used in escape codes.
*/
function IvCoreFlatten(object) {
var result = {}, temp, value;
for(var key in object) {
if(typeof (value = object[key]) === 'object') {
temp = IvCoreFlatten(value);
for(var x in temp) { 
result[key + '.' + x] = temp[x]; 
}
} else { 
result[key] = value; 
}
}
return result;
};


/**
* Returns true, if all language files have been loaded.
*/
function IvCoreIsReady() {
return IAVRA.Param.LocalizationCore.IvCoreLanguages.every(function(lang) { 
return !!IvCoreData[lang]; 
}); 
};

/**
* Recursively replaces all escape codes in the text with the localized data. Note, that we also do an implicit
* conversion of the given data to String, since otherwise the function would crash, when invoked with a number.
*/
var IvCoreReplace = function(text) {
if(text === undefined || text === null) { 
return text; 
}
var f = true;
var d = IvCoreData[IvCoreLang]; 
var r = '' + text;
while(f) { 
f = false; 
r = r.replace(IvCoreRegex, function(m, k) { 
f = true; return d[k]; }); 
}
return r;
};

//=============================================================================
// IAVRA.I18N
//=============================================================================

IAVRA.I18N = {
/**
* Localizes a given text. Can be used, when the automatic localization happens too late (or not at all):
*/
localize: function(text) { 
return IvCoreReplace(text); 
},

/**
* Returns the list of all registered languages. Can be used to create an option menu or similar.
*/
languages: function() { 
return IAVRA.Param.LocalizationCore.IvCoreLanguages; 
}
};

/**
* Property used to read and set the current language. If the given value wasn't registered in the "Languages"
* plugin parameter, fallback to the first language, instead. Also, when changing the language, we need to update
* the document title, since the game title might be localized.
*/
Object.defineProperty(IAVRA.I18N, 'language', {
get: function() { 
return IvCoreLang; 
},
set: function(value) {
IvCoreLang = IAVRA.Param.LocalizationCore.IvCoreLanguages.contains(value) ? value : IAVRA.Param.LocalizationCore.IvCoreLanguages[0];
Scene_Boot.prototype.updateDocumentTitle();
}
});

//=============================================================================
// Scene_Boot(RPG SCENE)
//=============================================================================

/**
* When creating Scene_Boot, also start loading all language files to initialize the plugin.
*/
IAVRA.Scene_Boot_create = Scene_Boot.prototype.create;
Scene_Boot.prototype.create = function() {
IAVRA.Scene_Boot_create.call(this);
IvCoreInitialize();
};

/**
* Also wait, until all language data has been read.
*/
IAVRA.Scene_Boot_isReady = Scene_Boot.prototype.isReady;
Scene_Boot.prototype.isReady = function() {
return IvCoreIsReady() && IAVRA.Scene_Boot_isReady.call(this);
};

/**
* We override this method, because we may have to convert the game title, before setting the document title.
* Make sure, that $dataSystem is already initialized, because an option menu will set the current language,
* before data has been loaded. But don't worry, the correct title will be set during Scene_Boot.start.
*/
IAVRA.Scene_Boot_updateDocumentTitle = Scene_Boot.prototype.updateDocumentTitle;
Scene_Boot.prototype.updateDocumentTitle = function() {
if($dataSystem) { document.title = IvCoreReplace($dataSystem.gameTitle); }
};

//=============================================================================
// Window_Base(RPG WINDOWS)
//=============================================================================

/**
* Set a marker indicating, that we are currently inside drawTextEx. This marker is not threadsafe, so the
* plugin likely won't work with other plugins, that are asynchronously drawing text.
*/
IAVRA.Window_Base_drawTextEx = Window_Base.prototype.drawTextEx;
Window_Base.prototype.drawTextEx = function(text, x, y) {
IvCoreinDrawTextEx = true;
IAVRA.result = IAVRA.Window_Base_drawTextEx.call(this, text, x, y);
IvCoreinDrawTextEx = false;
return IAVRA.result;
};
/**
* Replaces all escape codes, before AND after converting escape characters. We have to do this twice, because
* otherwise we would miss escape codes in character names, that have been added via escape characters.
*/
IAVRA.Window_Base_convertEscapeCharacters = Window_Base.prototype.convertEscapeCharacters;
Window_Base.prototype.convertEscapeCharacters = function(text) {
return IvCoreReplace(IAVRA.Window_Base_convertEscapeCharacters.call(this, IvCoreReplace(text)));
};

//=============================================================================
// Bitmap(RPG CORE)
//=============================================================================

/**
* When drawing text, replace escape codes unless we are currently inside drawTextEx. Without this check, we
* would effectively call our replace method for every single character of the text given to drawTextEx.
*/
IAVRA.Bitmap_drawText = Bitmap.prototype.drawText;
Bitmap.prototype.drawText = function(text, x, y, maxWidth, lineHeight, align) {
IAVRA.Bitmap_drawText.call(this, IvCoreinDrawTextEx ? text : IvCoreReplace(text), x, y, maxWidth, lineHeight, align);
};
/**
* Measuring text is done, before drawing it, so escape codes won't be resolved, yet.
*/
IAVRA.Bitmap_measureTextWidth = Bitmap.prototype.measureTextWidth;
Bitmap.prototype.measureTextWidth = function(text) {
return IAVRA.Bitmap_measureTextWidth.call(this, IvCoreReplace(text));
};

//=============================================================================
// String(RPG CORE)
//=============================================================================
/**
* Needs to be aliased, so the state and skill messages (among others) works correctly.
*/
IAVRA.String_format = String.prototype.format;
String.prototype.format = function() {
return IAVRA.String_format.apply(IvCoreReplace(this), arguments);
};

// A FIX FOR WINDOW_NAME EDIT MADE BY IAVRA FOR SOMEONE IN THE THREAD.
IAVRA.Window_NameEdit_initialize = Window_NameEdit.prototype.initialize;
Window_NameEdit.prototype.initialize = function(actor, maxLength) {
actor._name = IAVRA.I18N.localize(actor._name);
IAVRA.Window_NameEdit_initialize.call(this, actor, maxLength);
};

//=============================================================================
// LOCALIZATION MENU
//=============================================================================

/**
* Returns the previous language in the list or the last language, if the current one is already the first.
*/
// Eliaquim: I'V changed the behaviour of this to fix a bug.
function IvCorePrevLanguage(language) {
var languages = IAVRA.I18N.languages();
var index = languages.indexOf(language) - 1;
return languages[index < 0 ? languages.length - 1 : index];
};
/**
* Returns the next language in the list or the first language, of the current one is already the last.
*/
// Eliaquim: I'V changed the behaviour of this to fix a bug.
function IvCoreNextLanguage(language) {
var languages = IAVRA.I18N.languages();
var index = languages.indexOf(language) + 1;
return languages[index >= languages.length ? 0 : index];

};

/**
* Applies the given language and returns it or the default fallback, if the given value was invalid. We don't
* directly change the language, after it was selected in the options menu, because otherwise you would see a
* mixed of different languages, while changing options.
*/
function IvCoreApplyLanguage(language) {
IAVRA.I18N.language = language;
return IAVRA.I18N.language;
};

//=============================================================================
// Window_Options
//=============================================================================

// Eliaquim: I'v add this to fix a bug.
IAVRA.Window_Options_initialize = Window_Options.prototype.initialize;
Window_Options.prototype.initialize = function() {
ConfigManager.language = IAVRA.I18N.language;
IAVRA.Window_Options_initialize.call(this);
};

IAVRA.Window_Options_makeCommandList = Window_Options.prototype.makeCommandList;
Window_Options.prototype.makeCommandList = function() {
IAVRA.Window_Options_makeCommandList.call(this);
//this.addCommand(IAVRA.Param.LocalizationCore.IvCoreLabelOption, 'language');
};

IAVRA.Window_Options_statusText = Window_Options.prototype.statusText;
Window_Options.prototype.statusText = function(index) {
var symbol = this.commandSymbol(index);
var value = this.getConfigValue(symbol);
return symbol === 'language' ? IAVRA.Param.LocalizationCore.IvCoreLabelLanguages[value] : IAVRA.Window_Options_statusText.call(this, index);
};

IAVRA.Window_Options_processOk = Window_Options.prototype.processOk;
Window_Options.prototype.processOk = function() {
var symbol = this.commandSymbol(this.index());
var value = this.getConfigValue(symbol);
if(symbol === 'language') { 
return this.changeValue(symbol, IvCoreNextLanguage(value)); 
}
IAVRA.Window_Options_processOk.call(this);
};

IAVRA.Window_Options_cursorLeft = Window_Options.prototype.cursorLeft;
Window_Options.prototype.cursorLeft = function(wrap) {
var symbol = this.commandSymbol(this.index());
var value = this.getConfigValue(symbol);
if(symbol === 'language') { 
return this.changeValue(symbol, IvCorePrevLanguage(value));
}
IAVRA.Window_Options_cursorLeft.call(this, wrap);
};

IAVRA.Window_Options_cursorRight = Window_Options.prototype.cursorRight;
Window_Options.prototype.cursorRight = function(wrap) {
var symbol = this.commandSymbol(this.index());
var value = this.getConfigValue(symbol);
if(symbol === 'language') { 
return this.changeValue(symbol, IvCoreNextLanguage(value)); 
}
IAVRA.Window_Options_cursorRight.call(this, wrap);
};

//=============================================================================
// ConfigManager
//=============================================================================

/**
* The current language. Automatically gets kept in sync with the core plugin, whenever the current language
* is written to or read from the config savefile.
*/
var IvCoreLanguage;

/**
* Store the currently selected language in the config savefile, so it's persisted between games. Before
* saving the language, it gets applied, so we can correctly fallback to the default language, if needed.
*/
IAVRA.ConfigManager_makeData = ConfigManager.makeData;
ConfigManager.makeData = function() {
var config = IAVRA.ConfigManager_makeData.call(this);
config.language = this.language = IvCoreApplyLanguage(this.language);
return config;
};
/**
* Load the current language from the config savefile. Also, apply it, so we can display the correct language
* and fallback to the default language, if the savefile entry doesn't exist or is invalid.
*/
IAVRA.ConfigManager_applyData = ConfigManager.applyData;
ConfigManager.applyData = function(config) {
IAVRA.ConfigManager_applyData.call(this, config);
this.language = IvCoreApplyLanguage(config.language);
};
/**
* Read and set the current language. Language is managed locally, because otherwise we would experience some
* strange sideffects in the options menu.
*/
Object.defineProperty(IAVRA, 'language', {
get: function() { 
return IvCoreLanguage; 
},
set: function(value) { 
IvCoreLanguage = value; 
}
});